linux-kernel.vger.kernel.org archive mirror
 help / color / mirror / Atom feed
* [PATCH v12 0/10] Support Write-Through mapping on x86
@ 2015-06-01 19:36 Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 1/10] x86, mm, pat: Set WT to PA7 slot of PAT MSR Toshi Kani
                   ` (10 more replies)
  0 siblings, 11 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch

This patchset adds support of Write-Through (WT) mapping on x86.
The study below shows that using WT mapping may be useful for
non-volatile memory.

http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf

The patchset consists of the following changes.
 - Patch 1/10 to 6/10 add ioremap_wt()
 - Patch 7/10 adds pgprot_writethrough()
 - Patch 8/10 to 9/10 add set_memory_wt()
 - Patch 10/10 changes the pmem driver to call ioremap_wt()

All new/modified interfaces have been tested.

The patchset is based on:
git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2

---
v12:
- Rebased to bp/bp.git#tip-mm-2 and resolved minor conflicts. 
- Dropped cleanup patches 1-2 as redone by Boris.

v11:
- Reordered the refactor changes from patch 10-11 to 1-2.
  (Borislav Petkov)
- Changed BUG() to panic(). (Borislav Petkov)
- Rebased to tip/master and resolved conflicts. 

v10:
- Removed ioremap_writethrough(). (Thomas Gleixner)
- Clarified and cleaned up multiple comments and functions.
  (Thomas Gleixner) 
- Changed ioremap_change_attr() to accept the WT type.

v9:
- Changed to export the set_xxx_wt() interfaces with GPL.
  (Ingo Molnar)
- Changed is_new_memtype_allowed() to handle WT cases.
- Changed arch-specific io.h to define ioremap_wt().
- Changed the pmem driver to use ioremap_wt().
- Rebased to 4.1-rc3 and resolved minor conflicts.

v8:
- Rebased to 4.0-rc1 and resolved conflicts with 9d34cfdf4 in
  patch 5/7.

v7:
- Rebased to 3.19-rc3 as Juergen's patchset for the PAT management
  has been accepted.

v6:
- Dropped the patch moving [set|get]_page_memtype() to pat.c
  since the tip branch already has this change.
- Fixed an issue when CONFIG_X86_PAT is not defined.

v5:
- Clarified comment of why using slot 7. (Andy Lutomirski,
  Thomas Gleixner)
- Moved [set|get]_page_memtype() to pat.c. (Thomas Gleixner)
- Removed BUG() from set_page_memtype(). (Thomas Gleixner)

v4:
- Added set_memory_wt() by adding WT support of regular memory.

v3:
- Dropped the set_memory_wt() patch. (Andy Lutomirski)
- Refactored the !pat_enabled handling. (H. Peter Anvin,
  Andy Lutomirski)
- Added the picture of PTE encoding. (Konrad Rzeszutek Wilk)

v2:
- Changed WT to use slot 7 of the PAT MSR. (H. Peter Anvin,
  Andy Lutomirski)
- Changed to have conservative checks to exclude all Pentium 2, 3,
  M, and 4 families. (Ingo Molnar, Henrique de Moraes Holschuh,
  Andy Lutomirski)
- Updated documentation to cover WT interfaces and usages.
  (Andy Lutomirski, Yigal Korman)

---
Toshi Kani (10):
 1/10 x86, mm, pat: Set WT to PA7 slot of PAT MSR
 2/10 x86, mm, pat: Change reserve_memtype() for WT
 3/10 x86, asm: Change is_new_memtype_allowed() for WT
 4/10 x86, mm, asm-gen: Add ioremap_wt() for WT
 5/10 arch/*/asm/io.h: Add ioremap_wt() to all architectures
 6/10 video/fbdev, asm/io.h: Remove ioremap_writethrough()
 7/10 x86, mm, pat: Add pgprot_writethrough() for WT
 8/10 x86, mm, asm: Add WT support to set_page_memtype()
 9/10 x86, mm: Add set_memory_wt() for WT
10/10 drivers/block/pmem: Map NVDIMM with ioremap_wt()

---
 Documentation/x86/pat.txt            |  13 ++--
 arch/arc/include/asm/io.h            |   1 +
 arch/arm/include/asm/io.h            |   1 +
 arch/arm64/include/asm/io.h          |   1 +
 arch/avr32/include/asm/io.h          |   1 +
 arch/frv/include/asm/io.h            |   4 +-
 arch/m32r/include/asm/io.h           |   1 +
 arch/m68k/include/asm/io_mm.h        |   4 +-
 arch/m68k/include/asm/io_no.h        |   4 +-
 arch/metag/include/asm/io.h          |   3 +
 arch/microblaze/include/asm/io.h     |   2 +-
 arch/mn10300/include/asm/io.h        |   1 +
 arch/nios2/include/asm/io.h          |   1 +
 arch/s390/include/asm/io.h           |   1 +
 arch/sparc/include/asm/io_32.h       |   1 +
 arch/sparc/include/asm/io_64.h       |   1 +
 arch/tile/include/asm/io.h           |   2 +-
 arch/x86/include/asm/cacheflush.h    |   6 +-
 arch/x86/include/asm/io.h            |   2 +
 arch/x86/include/asm/pgtable.h       |   8 ++-
 arch/x86/include/asm/pgtable_types.h |   3 +
 arch/x86/mm/ioremap.c                |  24 +++++++
 arch/x86/mm/pageattr.c               |  62 +++++++++++++----
 arch/x86/mm/pat.c                    | 126 +++++++++++++++++++++++++----------
 arch/xtensa/include/asm/io.h         |   1 +
 drivers/block/pmem.c                 |   4 +-
 drivers/video/fbdev/amifb.c          |   4 +-
 drivers/video/fbdev/atafb.c          |   3 +-
 drivers/video/fbdev/hpfb.c           |   4 +-
 include/asm-generic/io.h             |   9 +++
 include/asm-generic/iomap.h          |   4 ++
 include/asm-generic/pgtable.h        |   4 ++
 32 files changed, 240 insertions(+), 66 deletions(-)

^ permalink raw reply	[flat|nested] 17+ messages in thread

* [PATCH v12 1/10] x86, mm, pat: Set WT to PA7 slot of PAT MSR
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 2/10] x86, mm, pat: Change reserve_memtype() for WT Toshi Kani
                   ` (9 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

This patch sets WT to the PA7 slot in the PAT MSR when the processor
is not affected by the PAT errata.  The PA7 slot is chosen to improve
robustness in the presence of errata that might cause the high PAT bit
to be ignored.  This way a buggy PA7 slot access will hit the PA3 slot,
which is UC, so at worst we lose performance without causing a correctness
issue.

The following Intel processors are affected by the PAT errata.

   errata               cpuid
   ----------------------------------------------------
   Pentium 2, A52       family 0x6, model 0x5
   Pentium 3, E27       family 0x6, model 0x7, 0x8
   Pentium 3 Xenon, G26 family 0x6, model 0x7, 0x8, 0xa
   Pentium M, Y26       family 0x6, model 0x9
   Pentium M 90nm, X9   family 0x6, model 0xd
   Pentium 4, N46       family 0xf, model 0x0

Instead of making sharp boundary checks, this patch makes conservative
checks to exclude all Pentium 2, 3, M and 4 family processors.  For
such processors, _PAGE_CACHE_MODE_WT is redirected to UC- per the
default setup in __cachemode2pte_tbl[].

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Reviewed-by: Juergen Gross <jgross@suse.com>
Reviewed-by: Thomas Gleixner <tglx@linutronix.de>
---
 arch/x86/mm/pat.c |   57 ++++++++++++++++++++++++++++++++++++++++++++++-------
 1 file changed, 49 insertions(+), 8 deletions(-)

diff --git a/arch/x86/mm/pat.c b/arch/x86/mm/pat.c
index 83af01a..56c8b4d 100644
--- a/arch/x86/mm/pat.c
+++ b/arch/x86/mm/pat.c
@@ -401,6 +401,8 @@ static int pat_pagerange_is_ram(resource_size_t start, resource_size_t end)
 
 /*
  * For RAM pages, we use page flags to mark the pages with appropriate type.
+ * The page flags are limited to three types, WB, WC and UC-.
+ * WT and WP requests fail with -EINVAL, and UC gets redirected to UC-.
  * Here we do two pass:
  * - Find the memtype of all the pages in the range, look for any conflicts
  * - In case of no conflicts, set the new memtype for pages in the range
@@ -412,6 +414,13 @@ static int reserve_ram_pages_type(u64 start, u64 end,
 	struct page *page;
 	u64 pfn;
 
+	if ((req_type == _PAGE_CACHE_MODE_WT) ||
+	    (req_type == _PAGE_CACHE_MODE_WP)) {
+		if (new_type)
+			*new_type = _PAGE_CACHE_MODE_UC_MINUS;
+		return -EINVAL;
+	}
+
 	if (req_type == _PAGE_CACHE_MODE_UC) {
 		/* We do not support strong UC */
 		WARN_ON_ONCE(1);
@@ -461,6 +470,7 @@ static int free_ram_pages_type(u64 start, u64 end)
  * - _PAGE_CACHE_MODE_WC
  * - _PAGE_CACHE_MODE_UC_MINUS
  * - _PAGE_CACHE_MODE_UC
+ * - _PAGE_CACHE_MODE_WT
  *
  * If new_type is NULL, function will return an error if it cannot reserve the
  * region with req_type. If new_type is non-NULL, function will return

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 3/10] x86, asm: Change is_new_memtype_allowed() for WT
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 1/10] x86, mm, pat: Set WT to PA7 slot of PAT MSR Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 2/10] x86, mm, pat: Change reserve_memtype() for WT Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 4/10] x86, mm, asm-gen: Add ioremap_wt() " Toshi Kani
                   ` (7 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

__ioremap_caller() calls reserve_memtype() to set new_pcm
(existing map type if any), and then calls
is_new_memtype_allowed() to verify if converting to new_pcm
is allowed when pcm (request type) is different from new_pcm.

When WT is requested, the caller expects that writes are
ordered and uncached.  Therefore, this patch changes
is_new_memtype_allowed() to disallow the following cases.

 - If the request is WT, mapping type cannot be WB
 - If the request is WT, mapping type cannot be WC

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Reviewed-by: Thomas Gleixner <tglx@linutronix.de>
---
 arch/x86/include/asm/pgtable.h |    8 +++++++-
 1 file changed, 7 insertions(+), 1 deletion(-)

diff --git a/arch/x86/include/asm/pgtable.h b/arch/x86/include/asm/pgtable.h
index fe57e7a..2562e30 100644
--- a/arch/x86/include/asm/pgtable.h
+++ b/arch/x86/include/asm/pgtable.h
@@ -398,11 +398,17 @@ static inline int is_new_memtype_allowed(u64 paddr, unsigned long size,
 	 * requested memtype:
 	 * - request is uncached, return cannot be write-back
 	 * - request is write-combine, return cannot be write-back
+	 * - request is write-through, return cannot be write-back
+	 * - request is write-through, return cannot be write-combine
 	 */
 	if ((pcm == _PAGE_CACHE_MODE_UC_MINUS &&
 	     new_pcm == _PAGE_CACHE_MODE_WB) ||
 	    (pcm == _PAGE_CACHE_MODE_WC &&
-	     new_pcm == _PAGE_CACHE_MODE_WB)) {
+	     new_pcm == _PAGE_CACHE_MODE_WB) ||
+	    (pcm == _PAGE_CACHE_MODE_WT &&
+	     new_pcm == _PAGE_CACHE_MODE_WB) ||
+	    (pcm == _PAGE_CACHE_MODE_WT &&
+	     new_pcm == _PAGE_CACHE_MODE_WC)) {
 		return 0;
 	}
 

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 4/10] x86, mm, asm-gen: Add ioremap_wt() for WT
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (2 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 3/10] x86, asm: Change is_new_memtype_allowed() " Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 5/10] arch/*/asm/io.h: Add ioremap_wt() to all architectures Toshi Kani
                   ` (6 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

This patch adds ioremap_wt() for creating WT mapping on x86.
It follows the same model as ioremap_wc() for multi-architecture
support.  ARCH_HAS_IOREMAP_WT is defined in the x86 version of
io.h to indicate that ioremap_wt() is implemented on x86.

Also update the PAT documentation file to cover ioremap_wt().

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Reviewed-by: Konrad Rzeszutek Wilk <konrad.wilk@oracle.com>
Reviewed-by: Thomas Gleixner <tglx@linutronix.de>
---
 Documentation/x86/pat.txt   |    4 +++-
 arch/x86/include/asm/io.h   |    2 ++
 arch/x86/mm/ioremap.c       |   21 +++++++++++++++++++++
 include/asm-generic/io.h    |    9 +++++++++
 include/asm-generic/iomap.h |    4 ++++
 5 files changed, 39 insertions(+), 1 deletion(-)

diff --git a/Documentation/x86/pat.txt b/Documentation/x86/pat.txt
index 521bd8a..db0de6c 100644
--- a/Documentation/x86/pat.txt
+++ b/Documentation/x86/pat.txt
@@ -12,7 +12,7 @@ virtual addresses.
 
 PAT allows for different types of memory attributes. The most commonly used
 ones that will be supported at this time are Write-back, Uncached,
-Write-combined and Uncached Minus.
+Write-combined, Write-through and Uncached Minus.
 
 
 PAT APIs
@@ -40,6 +40,8 @@ ioremap_nocache        |    --    |    UC-     |       UC-        |
                        |          |            |                  |
 ioremap_wc             |    --    |    --      |       WC         |
                        |          |            |                  |
+ioremap_wt             |    --    |    --      |       WT         |
+                       |          |            |                  |
 set_memory_uc          |    UC-   |    --      |       --         |
  set_memory_wb         |          |            |                  |
                        |          |            |                  |
diff --git a/arch/x86/include/asm/io.h b/arch/x86/include/asm/io.h
index a2b9740..6c3a130 100644
--- a/arch/x86/include/asm/io.h
+++ b/arch/x86/include/asm/io.h
@@ -35,6 +35,7 @@
   */
 
 #define ARCH_HAS_IOREMAP_WC
+#define ARCH_HAS_IOREMAP_WT
 
 #include <linux/string.h>
 #include <linux/compiler.h>
@@ -321,6 +322,7 @@ extern void unxlate_dev_mem_ptr(phys_addr_t phys, void *addr);
 extern int ioremap_change_attr(unsigned long vaddr, unsigned long size,
 				enum page_cache_mode pcm);
 extern void __iomem *ioremap_wc(resource_size_t offset, unsigned long size);
+extern void __iomem *ioremap_wt(resource_size_t offset, unsigned long size);
 
 extern bool is_early_ioremap_ptep(pte_t *ptep);
 
diff --git a/arch/x86/mm/ioremap.c b/arch/x86/mm/ioremap.c
index cc0f17c..07cd46a 100644
--- a/arch/x86/mm/ioremap.c
+++ b/arch/x86/mm/ioremap.c
@@ -172,6 +172,10 @@ static void __iomem *__ioremap_caller(resource_size_t phys_addr,
 		prot = __pgprot(pgprot_val(prot) |
 				cachemode2protval(_PAGE_CACHE_MODE_WC));
 		break;
+	case _PAGE_CACHE_MODE_WT:
+		prot = __pgprot(pgprot_val(prot) |
+				cachemode2protval(_PAGE_CACHE_MODE_WT));
+		break;
 	case _PAGE_CACHE_MODE_WB:
 		break;
 	}
@@ -297,6 +301,23 @@ void __iomem *ioremap_wc(resource_size_t phys_addr, unsigned long size)
 }
 EXPORT_SYMBOL(ioremap_wc);
 
+/**
+ * ioremap_wt	-	map memory into CPU space write through
+ * @phys_addr:	bus address of the memory
+ * @size:	size of the resource to map
+ *
+ * This version of ioremap ensures that the memory is marked write through.
+ * Write through stores data into memory while keeping the cache up-to-date.
+ *
+ * Must be freed with iounmap.
+ */
+void __iomem *ioremap_wt(resource_size_t phys_addr, unsigned long size)
+{
+	return __ioremap_caller(phys_addr, size, _PAGE_CACHE_MODE_WT,
+					__builtin_return_address(0));
+}
+EXPORT_SYMBOL(ioremap_wt);
+
 void __iomem *ioremap_cache(resource_size_t phys_addr, unsigned long size)
 {
 	return __ioremap_caller(phys_addr, size, _PAGE_CACHE_MODE_WB,
diff --git a/include/asm-generic/io.h b/include/asm-generic/io.h
index 90ccba7..f56094c 100644
--- a/include/asm-generic/io.h
+++ b/include/asm-generic/io.h
@@ -785,8 +785,17 @@ static inline void __iomem *ioremap_wc(phys_addr_t offset, size_t size)
 }
 #endif
 
+#ifndef ioremap_wt
+#define ioremap_wt ioremap_wt
+static inline void __iomem *ioremap_wt(phys_addr_t offset, size_t size)
+{
+	return ioremap_nocache(offset, size);
+}
+#endif
+
 #ifndef iounmap
 #define iounmap iounmap
+
 static inline void iounmap(void __iomem *addr)
 {
 }
diff --git a/include/asm-generic/iomap.h b/include/asm-generic/iomap.h
index 1b41011..d8f8622 100644
--- a/include/asm-generic/iomap.h
+++ b/include/asm-generic/iomap.h
@@ -66,6 +66,10 @@ extern void ioport_unmap(void __iomem *);
 #define ioremap_wc ioremap_nocache
 #endif
 
+#ifndef ARCH_HAS_IOREMAP_WT
+#define ioremap_wt ioremap_nocache
+#endif
+
 #ifdef CONFIG_PCI
 /* Destroy a virtual mapping cookie for a PCI BAR (memory or IO) */
 struct pci_dev;

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 5/10] arch/*/asm/io.h: Add ioremap_wt() to all architectures
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (3 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 4/10] x86, mm, asm-gen: Add ioremap_wt() " Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough() Toshi Kani
                   ` (5 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

This patch adds ioremap_wt() to all arch-specific asm/io.h which
define ioremap_wc() locally.  These arch-specific asm/io.h do not
include <asm-generic/iomap.h>.  Some of them include
<asm-generic/io.h>, but ioremap_wt() is defined for consistency
since they define all ioremap_xxx locally.

ioremap_wt() is defined indentical to ioremap_nocache() to all
architectures without WT support.

frv and m68k already have ioremap_writethrough().  This patch
implements ioremap_wt() indetical to ioremap_writethrough() and
defines ARCH_HAS_IOREMAP_WT in both architectures.

This patch allows generic drivers to use ioremap_wt().

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
---
 arch/arc/include/asm/io.h        |    1 +
 arch/arm/include/asm/io.h        |    1 +
 arch/arm64/include/asm/io.h      |    1 +
 arch/avr32/include/asm/io.h      |    1 +
 arch/frv/include/asm/io.h        |    7 +++++++
 arch/m32r/include/asm/io.h       |    1 +
 arch/m68k/include/asm/io_mm.h    |    7 +++++++
 arch/m68k/include/asm/io_no.h    |    6 ++++++
 arch/metag/include/asm/io.h      |    3 +++
 arch/microblaze/include/asm/io.h |    1 +
 arch/mn10300/include/asm/io.h    |    1 +
 arch/nios2/include/asm/io.h      |    1 +
 arch/s390/include/asm/io.h       |    1 +
 arch/sparc/include/asm/io_32.h   |    1 +
 arch/sparc/include/asm/io_64.h   |    1 +
 arch/tile/include/asm/io.h       |    1 +
 arch/xtensa/include/asm/io.h     |    1 +
 17 files changed, 36 insertions(+)

diff --git a/arch/arc/include/asm/io.h b/arch/arc/include/asm/io.h
index cabd518..7cc4ced 100644
--- a/arch/arc/include/asm/io.h
+++ b/arch/arc/include/asm/io.h
@@ -20,6 +20,7 @@ extern void iounmap(const void __iomem *addr);
 
 #define ioremap_nocache(phy, sz)	ioremap(phy, sz)
 #define ioremap_wc(phy, sz)		ioremap(phy, sz)
+#define ioremap_wt(phy, sz)		ioremap(phy, sz)
 
 /* Change struct page to physical address */
 #define page_to_phys(page)		(page_to_pfn(page) << PAGE_SHIFT)
diff --git a/arch/arm/include/asm/io.h b/arch/arm/include/asm/io.h
index db58deb..1b7677d 100644
--- a/arch/arm/include/asm/io.h
+++ b/arch/arm/include/asm/io.h
@@ -336,6 +336,7 @@ extern void _memset_io(volatile void __iomem *, int, size_t);
 #define ioremap_nocache(cookie,size)	__arm_ioremap((cookie), (size), MT_DEVICE)
 #define ioremap_cache(cookie,size)	__arm_ioremap((cookie), (size), MT_DEVICE_CACHED)
 #define ioremap_wc(cookie,size)		__arm_ioremap((cookie), (size), MT_DEVICE_WC)
+#define ioremap_wt(cookie,size)		__arm_ioremap((cookie), (size), MT_DEVICE)
 #define iounmap				__arm_iounmap
 
 /*
diff --git a/arch/arm64/include/asm/io.h b/arch/arm64/include/asm/io.h
index 540f7c0..7116d39 100644
--- a/arch/arm64/include/asm/io.h
+++ b/arch/arm64/include/asm/io.h
@@ -170,6 +170,7 @@ extern void __iomem *ioremap_cache(phys_addr_t phys_addr, size_t size);
 #define ioremap(addr, size)		__ioremap((addr), (size), __pgprot(PROT_DEVICE_nGnRE))
 #define ioremap_nocache(addr, size)	__ioremap((addr), (size), __pgprot(PROT_DEVICE_nGnRE))
 #define ioremap_wc(addr, size)		__ioremap((addr), (size), __pgprot(PROT_NORMAL_NC))
+#define ioremap_wt(addr, size)		__ioremap((addr), (size), __pgprot(PROT_DEVICE_nGnRE))
 #define iounmap				__iounmap
 
 /*
diff --git a/arch/avr32/include/asm/io.h b/arch/avr32/include/asm/io.h
index 4f5ec2b..e998ff5 100644
--- a/arch/avr32/include/asm/io.h
+++ b/arch/avr32/include/asm/io.h
@@ -296,6 +296,7 @@ extern void __iounmap(void __iomem *addr);
 	__iounmap(addr)
 
 #define ioremap_wc ioremap_nocache
+#define ioremap_wt ioremap_nocache
 
 #define cached(addr) P1SEGADDR(addr)
 #define uncached(addr) P2SEGADDR(addr)
diff --git a/arch/frv/include/asm/io.h b/arch/frv/include/asm/io.h
index 0b78bc8..1fe98fe 100644
--- a/arch/frv/include/asm/io.h
+++ b/arch/frv/include/asm/io.h
@@ -17,6 +17,8 @@
 
 #ifdef __KERNEL__
 
+#define ARCH_HAS_IOREMAP_WT
+
 #include <linux/types.h>
 #include <asm/virtconvert.h>
 #include <asm/string.h>
@@ -270,6 +272,11 @@ static inline void __iomem *ioremap_writethrough(unsigned long physaddr, unsigne
 	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
 }
 
+static inline void __iomem *ioremap_wt(unsigned long physaddr, unsigned long size)
+{
+	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
+}
+
 static inline void __iomem *ioremap_fullcache(unsigned long physaddr, unsigned long size)
 {
 	return __ioremap(physaddr, size, IOMAP_FULL_CACHING);
diff --git a/arch/m32r/include/asm/io.h b/arch/m32r/include/asm/io.h
index 9cc00db..0c3f25e 100644
--- a/arch/m32r/include/asm/io.h
+++ b/arch/m32r/include/asm/io.h
@@ -68,6 +68,7 @@ static inline void __iomem *ioremap(unsigned long offset, unsigned long size)
 extern void iounmap(volatile void __iomem *addr);
 #define ioremap_nocache(off,size) ioremap(off,size)
 #define ioremap_wc ioremap_nocache
+#define ioremap_wt ioremap_nocache
 
 /*
  * IO bus memory addresses are also 1:1 with the physical address
diff --git a/arch/m68k/include/asm/io_mm.h b/arch/m68k/include/asm/io_mm.h
index 8955b40..7c12138 100644
--- a/arch/m68k/include/asm/io_mm.h
+++ b/arch/m68k/include/asm/io_mm.h
@@ -20,6 +20,8 @@
 
 #ifdef __KERNEL__
 
+#define ARCH_HAS_IOREMAP_WT
+
 #include <linux/compiler.h>
 #include <asm/raw_io.h>
 #include <asm/virtconvert.h>
@@ -470,6 +472,11 @@ static inline void __iomem *ioremap_writethrough(unsigned long physaddr,
 {
 	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
 }
+static inline void __iomem *ioremap_wt(unsigned long physaddr,
+					 unsigned long size)
+{
+	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
+}
 static inline void __iomem *ioremap_fullcache(unsigned long physaddr,
 				      unsigned long size)
 {
diff --git a/arch/m68k/include/asm/io_no.h b/arch/m68k/include/asm/io_no.h
index a93c8cd..5fff9a2 100644
--- a/arch/m68k/include/asm/io_no.h
+++ b/arch/m68k/include/asm/io_no.h
@@ -3,6 +3,8 @@
 
 #ifdef __KERNEL__
 
+#define ARCH_HAS_IOREMAP_WT
+
 #include <asm/virtconvert.h>
 #include <asm-generic/iomap.h>
 
@@ -157,6 +159,10 @@ static inline void *ioremap_writethrough(unsigned long physaddr, unsigned long s
 {
 	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
 }
+static inline void *ioremap_wt(unsigned long physaddr, unsigned long size)
+{
+	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
+}
 static inline void *ioremap_fullcache(unsigned long physaddr, unsigned long size)
 {
 	return __ioremap(physaddr, size, IOMAP_FULL_CACHING);
diff --git a/arch/metag/include/asm/io.h b/arch/metag/include/asm/io.h
index d5779b0..9890f21 100644
--- a/arch/metag/include/asm/io.h
+++ b/arch/metag/include/asm/io.h
@@ -160,6 +160,9 @@ extern void __iounmap(void __iomem *addr);
 #define ioremap_wc(offset, size)                \
 	__ioremap((offset), (size), _PAGE_WR_COMBINE)
 
+#define ioremap_wt(offset, size)                \
+	__ioremap((offset), (size), 0)
+
 #define iounmap(addr)                           \
 	__iounmap(addr)
 
diff --git a/arch/microblaze/include/asm/io.h b/arch/microblaze/include/asm/io.h
index 940f5fc..ec3da11 100644
--- a/arch/microblaze/include/asm/io.h
+++ b/arch/microblaze/include/asm/io.h
@@ -43,6 +43,7 @@ extern void __iomem *ioremap(phys_addr_t address, unsigned long size);
 #define ioremap_nocache(addr, size)		ioremap((addr), (size))
 #define ioremap_fullcache(addr, size)		ioremap((addr), (size))
 #define ioremap_wc(addr, size)			ioremap((addr), (size))
+#define ioremap_wt(addr, size)			ioremap((addr), (size))
 
 #endif /* CONFIG_MMU */
 
diff --git a/arch/mn10300/include/asm/io.h b/arch/mn10300/include/asm/io.h
index cc4a2ba..07c5b4a 100644
--- a/arch/mn10300/include/asm/io.h
+++ b/arch/mn10300/include/asm/io.h
@@ -282,6 +282,7 @@ static inline void __iomem *ioremap_nocache(unsigned long offset, unsigned long
 }
 
 #define ioremap_wc ioremap_nocache
+#define ioremap_wt ioremap_nocache
 
 static inline void iounmap(void __iomem *addr)
 {
diff --git a/arch/nios2/include/asm/io.h b/arch/nios2/include/asm/io.h
index 6e24d7c..c5a62da 100644
--- a/arch/nios2/include/asm/io.h
+++ b/arch/nios2/include/asm/io.h
@@ -46,6 +46,7 @@ static inline void iounmap(void __iomem *addr)
 }
 
 #define ioremap_wc ioremap_nocache
+#define ioremap_wt ioremap_nocache
 
 /* Pages to physical address... */
 #define page_to_phys(page)	virt_to_phys(page_to_virt(page))
diff --git a/arch/s390/include/asm/io.h b/arch/s390/include/asm/io.h
index 30fd5c8..cb5fdf3 100644
--- a/arch/s390/include/asm/io.h
+++ b/arch/s390/include/asm/io.h
@@ -29,6 +29,7 @@ void unxlate_dev_mem_ptr(phys_addr_t phys, void *addr);
 
 #define ioremap_nocache(addr, size)	ioremap(addr, size)
 #define ioremap_wc			ioremap_nocache
+#define ioremap_wt			ioremap_nocache
 
 static inline void __iomem *ioremap(unsigned long offset, unsigned long size)
 {
diff --git a/arch/sparc/include/asm/io_32.h b/arch/sparc/include/asm/io_32.h
index 407ac14..57f26c3 100644
--- a/arch/sparc/include/asm/io_32.h
+++ b/arch/sparc/include/asm/io_32.h
@@ -129,6 +129,7 @@ static inline void sbus_memcpy_toio(volatile void __iomem *dst,
 void __iomem *ioremap(unsigned long offset, unsigned long size);
 #define ioremap_nocache(X,Y)	ioremap((X),(Y))
 #define ioremap_wc(X,Y)		ioremap((X),(Y))
+#define ioremap_wt(X,Y)		ioremap((X),(Y))
 void iounmap(volatile void __iomem *addr);
 
 /* Create a virtual mapping cookie for an IO port range */
diff --git a/arch/sparc/include/asm/io_64.h b/arch/sparc/include/asm/io_64.h
index 50d4840..c32fa3f 100644
--- a/arch/sparc/include/asm/io_64.h
+++ b/arch/sparc/include/asm/io_64.h
@@ -402,6 +402,7 @@ static inline void __iomem *ioremap(unsigned long offset, unsigned long size)
 
 #define ioremap_nocache(X,Y)		ioremap((X),(Y))
 #define ioremap_wc(X,Y)			ioremap((X),(Y))
+#define ioremap_wt(X,Y)			ioremap((X),(Y))
 
 static inline void iounmap(volatile void __iomem *addr)
 {
diff --git a/arch/tile/include/asm/io.h b/arch/tile/include/asm/io.h
index 6ef4eca..9c3d950 100644
--- a/arch/tile/include/asm/io.h
+++ b/arch/tile/include/asm/io.h
@@ -54,6 +54,7 @@ extern void iounmap(volatile void __iomem *addr);
 
 #define ioremap_nocache(physaddr, size)		ioremap(physaddr, size)
 #define ioremap_wc(physaddr, size)		ioremap(physaddr, size)
+#define ioremap_wt(physaddr, size)		ioremap(physaddr, size)
 #define ioremap_writethrough(physaddr, size)	ioremap(physaddr, size)
 #define ioremap_fullcache(physaddr, size)	ioremap(physaddr, size)
 
diff --git a/arch/xtensa/include/asm/io.h b/arch/xtensa/include/asm/io.h
index fe1600a..c39bb6e 100644
--- a/arch/xtensa/include/asm/io.h
+++ b/arch/xtensa/include/asm/io.h
@@ -59,6 +59,7 @@ static inline void __iomem *ioremap_cache(unsigned long offset,
 }
 
 #define ioremap_wc ioremap_nocache
+#define ioremap_wt ioremap_nocache
 
 static inline void __iomem *ioremap(unsigned long offset, unsigned long size)
 {

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough()
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (4 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 5/10] arch/*/asm/io.h: Add ioremap_wt() to all architectures Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-02  7:06   ` Geert Uytterhoeven
  2015-06-01 19:36 ` [PATCH v12 7/10] x86, mm, pat: Add pgprot_writethrough() for WT Toshi Kani
                   ` (4 subsequent siblings)
  10 siblings, 1 reply; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

This patch removes the callers of ioremap_writethrough() by
replacing them with ioremap_wt() in three drivers under
drivers/video/fbdev.  It then removes ioremap_writethrough()
defined in some architecture's asm/io.h, frv, m68k, microblaze,
and tile.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
---
 arch/frv/include/asm/io.h        |    5 -----
 arch/m68k/include/asm/io_mm.h    |    5 -----
 arch/m68k/include/asm/io_no.h    |    4 ----
 arch/microblaze/include/asm/io.h |    1 -
 arch/tile/include/asm/io.h       |    1 -
 drivers/video/fbdev/amifb.c      |    4 ++--
 drivers/video/fbdev/atafb.c      |    3 +--
 drivers/video/fbdev/hpfb.c       |    4 ++--
 8 files changed, 5 insertions(+), 22 deletions(-)

diff --git a/arch/frv/include/asm/io.h b/arch/frv/include/asm/io.h
index 1fe98fe..a31b63e 100644
--- a/arch/frv/include/asm/io.h
+++ b/arch/frv/include/asm/io.h
@@ -267,11 +267,6 @@ static inline void __iomem *ioremap_nocache(unsigned long physaddr, unsigned lon
 	return __ioremap(physaddr, size, IOMAP_NOCACHE_SER);
 }
 
-static inline void __iomem *ioremap_writethrough(unsigned long physaddr, unsigned long size)
-{
-	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
-}
-
 static inline void __iomem *ioremap_wt(unsigned long physaddr, unsigned long size)
 {
 	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
diff --git a/arch/m68k/include/asm/io_mm.h b/arch/m68k/include/asm/io_mm.h
index 7c12138..618c85d3 100644
--- a/arch/m68k/include/asm/io_mm.h
+++ b/arch/m68k/include/asm/io_mm.h
@@ -467,11 +467,6 @@ static inline void __iomem *ioremap_nocache(unsigned long physaddr, unsigned lon
 {
 	return __ioremap(physaddr, size, IOMAP_NOCACHE_SER);
 }
-static inline void __iomem *ioremap_writethrough(unsigned long physaddr,
-					 unsigned long size)
-{
-	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
-}
 static inline void __iomem *ioremap_wt(unsigned long physaddr,
 					 unsigned long size)
 {
diff --git a/arch/m68k/include/asm/io_no.h b/arch/m68k/include/asm/io_no.h
index 5fff9a2..ad7bd40 100644
--- a/arch/m68k/include/asm/io_no.h
+++ b/arch/m68k/include/asm/io_no.h
@@ -155,10 +155,6 @@ static inline void *ioremap_nocache(unsigned long physaddr, unsigned long size)
 {
 	return __ioremap(physaddr, size, IOMAP_NOCACHE_SER);
 }
-static inline void *ioremap_writethrough(unsigned long physaddr, unsigned long size)
-{
-	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
-}
 static inline void *ioremap_wt(unsigned long physaddr, unsigned long size)
 {
 	return __ioremap(physaddr, size, IOMAP_WRITETHROUGH);
diff --git a/arch/microblaze/include/asm/io.h b/arch/microblaze/include/asm/io.h
index ec3da11..39b6315 100644
--- a/arch/microblaze/include/asm/io.h
+++ b/arch/microblaze/include/asm/io.h
@@ -39,7 +39,6 @@ extern resource_size_t isa_mem_base;
 extern void iounmap(void __iomem *addr);
 
 extern void __iomem *ioremap(phys_addr_t address, unsigned long size);
-#define ioremap_writethrough(addr, size)	ioremap((addr), (size))
 #define ioremap_nocache(addr, size)		ioremap((addr), (size))
 #define ioremap_fullcache(addr, size)		ioremap((addr), (size))
 #define ioremap_wc(addr, size)			ioremap((addr), (size))
diff --git a/arch/tile/include/asm/io.h b/arch/tile/include/asm/io.h
index 9c3d950..dc61de1 100644
--- a/arch/tile/include/asm/io.h
+++ b/arch/tile/include/asm/io.h
@@ -55,7 +55,6 @@ extern void iounmap(volatile void __iomem *addr);
 #define ioremap_nocache(physaddr, size)		ioremap(physaddr, size)
 #define ioremap_wc(physaddr, size)		ioremap(physaddr, size)
 #define ioremap_wt(physaddr, size)		ioremap(physaddr, size)
-#define ioremap_writethrough(physaddr, size)	ioremap(physaddr, size)
 #define ioremap_fullcache(physaddr, size)	ioremap(physaddr, size)
 
 #define mmiowb()
diff --git a/drivers/video/fbdev/amifb.c b/drivers/video/fbdev/amifb.c
index 35f7900..ee3a703 100644
--- a/drivers/video/fbdev/amifb.c
+++ b/drivers/video/fbdev/amifb.c
@@ -3705,8 +3705,8 @@ default_chipset:
 	 * access the videomem with writethrough cache
 	 */
 	info->fix.smem_start = (u_long)ZTWO_PADDR(videomemory);
-	videomemory = (u_long)ioremap_writethrough(info->fix.smem_start,
-						   info->fix.smem_len);
+	videomemory = (u_long)ioremap_wt(info->fix.smem_start,
+					 info->fix.smem_len);
 	if (!videomemory) {
 		dev_warn(&pdev->dev,
 			 "Unable to map videomem cached writethrough\n");
diff --git a/drivers/video/fbdev/atafb.c b/drivers/video/fbdev/atafb.c
index cb9ee25..d6ce613 100644
--- a/drivers/video/fbdev/atafb.c
+++ b/drivers/video/fbdev/atafb.c
@@ -3185,8 +3185,7 @@ int __init atafb_init(void)
 		/* Map the video memory (physical address given) to somewhere
 		 * in the kernel address space.
 		 */
-		external_screen_base = ioremap_writethrough(external_addr,
-						     external_len);
+		external_screen_base = ioremap_wt(external_addr, external_len);
 		if (external_vgaiobase)
 			external_vgaiobase =
 			  (unsigned long)ioremap(external_vgaiobase, 0x10000);
diff --git a/drivers/video/fbdev/hpfb.c b/drivers/video/fbdev/hpfb.c
index a1b7e5f..9476d19 100644
--- a/drivers/video/fbdev/hpfb.c
+++ b/drivers/video/fbdev/hpfb.c
@@ -241,8 +241,8 @@ static int hpfb_init_one(unsigned long phys_base, unsigned long virt_base)
 	fb_info.fix.line_length = fb_width;
 	fb_height = (in_8(fb_regs + HPFB_FBHMSB) << 8) | in_8(fb_regs + HPFB_FBHLSB);
 	fb_info.fix.smem_len = fb_width * fb_height;
-	fb_start = (unsigned long)ioremap_writethrough(fb_info.fix.smem_start,
-						       fb_info.fix.smem_len);
+	fb_start = (unsigned long)ioremap_wt(fb_info.fix.smem_start,
+					     fb_info.fix.smem_len);
 	hpfb_defined.xres = (in_8(fb_regs + HPFB_DWMSB) << 8) | in_8(fb_regs + HPFB_DWLSB);
 	hpfb_defined.yres = (in_8(fb_regs + HPFB_DHMSB) << 8) | in_8(fb_regs + HPFB_DHLSB);
 	hpfb_defined.xres_virtual = hpfb_defined.xres;

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 7/10] x86, mm, pat: Add pgprot_writethrough() for WT
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (5 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough() Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 08/10] x86, mm, asm: Add WT support to set_page_memtype() Toshi Kani
                   ` (3 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

This patch adds pgprot_writethrough() for setting WT to a given
pgprot_t.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Reviewed-by: Konrad Rzeszutek Wilk <konrad.wilk@oracle.com>
Reviewed-by: Thomas Gleixner <tglx@linutronix.de>
---
 arch/x86/include/asm/pgtable_types.h |    3 +++
 arch/x86/mm/pat.c                    |    7 +++++++
 include/asm-generic/pgtable.h        |    4 ++++
 3 files changed, 14 insertions(+)

diff --git a/arch/x86/include/asm/pgtable_types.h b/arch/x86/include/asm/pgtable_types.h
index 78f0c8c..13f310b 100644
--- a/arch/x86/include/asm/pgtable_types.h
+++ b/arch/x86/include/asm/pgtable_types.h
@@ -367,6 +367,9 @@ extern int nx_enabled;
 #define pgprot_writecombine	pgprot_writecombine
 extern pgprot_t pgprot_writecombine(pgprot_t prot);
 
+#define pgprot_writethrough	pgprot_writethrough
+extern pgprot_t pgprot_writethrough(pgprot_t prot);
+
 /* Indicate that x86 has its own track and untrack pfn vma functions */
 #define __HAVE_PFNMAP_TRACKING
 
diff --git a/arch/x86/mm/pat.c b/arch/x86/mm/pat.c
index 56c8b4d..a89bcde 100644
--- a/arch/x86/mm/pat.c
+++ b/arch/x86/mm/pat.c
@@ -999,6 +999,13 @@ pgprot_t pgprot_writecombine(pgprot_t prot)
 }
 EXPORT_SYMBOL_GPL(pgprot_writecombine);
 
+pgprot_t pgprot_writethrough(pgprot_t prot)
+{
+	return __pgprot(pgprot_val(prot) |
+				cachemode2protval(_PAGE_CACHE_MODE_WT));
+}
+EXPORT_SYMBOL_GPL(pgprot_writethrough);
+
 #if defined(CONFIG_DEBUG_FS) && defined(CONFIG_X86_PAT)
 
 static struct memtype *memtype_get_idx(loff_t pos)
diff --git a/include/asm-generic/pgtable.h b/include/asm-generic/pgtable.h
index 39f1d6a..bd910ce 100644
--- a/include/asm-generic/pgtable.h
+++ b/include/asm-generic/pgtable.h
@@ -262,6 +262,10 @@ static inline int pmd_same(pmd_t pmd_a, pmd_t pmd_b)
 #define pgprot_writecombine pgprot_noncached
 #endif
 
+#ifndef pgprot_writethrough
+#define pgprot_writethrough pgprot_noncached
+#endif
+
 #ifndef pgprot_device
 #define pgprot_device pgprot_noncached
 #endif

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 08/10] x86, mm, asm: Add WT support to set_page_memtype()
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (6 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 7/10] x86, mm, pat: Add pgprot_writethrough() for WT Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 09/10] x86, mm: Add set_memory_wt() for WT Toshi Kani
                   ` (2 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

As set_memory_wb() calls free_ram_pages_type(), which then calls
set_page_memtype() with -1, _PGMT_DEFAULT is used for tracking
the WB type.  _PGMT_WB is defined but unused.  Hence, this patch
renames _PGMT_DEFAULT to _PGMT_WB to clarify the usage, and
releases the slot used by _PGMT_WB before.  free_ram_pages_type()
is changed to call set_page_memtype() with _PGMT_WB, and
get_page_memtype() returns _PAGE_CACHE_MODE_WB for _PGMT_WB.

This patch then defines _PGMT_WT to the released slot.  This enables
set_page_memtype() to track the WT type.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
---
 arch/x86/mm/pat.c |   60 +++++++++++++++++++++++++++--------------------------
 1 file changed, 30 insertions(+), 30 deletions(-)

diff --git a/arch/x86/mm/pat.c b/arch/x86/mm/pat.c
index a89bcde..2bb73ce 100644
--- a/arch/x86/mm/pat.c
+++ b/arch/x86/mm/pat.c
@@ -70,18 +70,22 @@ __setup("debugpat", pat_debug_setup);
 
 #ifdef CONFIG_X86_PAT
 /*
- * X86 PAT uses page flags WC and Uncached together to keep track of
- * memory type of pages that have backing page struct. X86 PAT supports 3
- * different memory types, _PAGE_CACHE_MODE_WB, _PAGE_CACHE_MODE_WC and
- * _PAGE_CACHE_MODE_UC_MINUS and fourth state where page's memory type has not
- * been changed from its default (value of -1 used to denote this).
- * Note we do not support _PAGE_CACHE_MODE_UC here.
+ * X86 PAT uses page flags arch_1 and uncached together to keep track of
+ * memory type of pages that have backing page struct.
+ *
+ * X86 PAT supports 4 different memory types:
+ *  - _PAGE_CACHE_MODE_WB
+ *  - _PAGE_CACHE_MODE_WC
+ *  - _PAGE_CACHE_MODE_UC_MINUS
+ *  - _PAGE_CACHE_MODE_WT
+ *
+ * _PAGE_CACHE_MODE_WB is the default type.
  */
 
-#define _PGMT_DEFAULT		0
+#define _PGMT_WB		0
 #define _PGMT_WC		(1UL << PG_arch_1)
 #define _PGMT_UC_MINUS		(1UL << PG_uncached)
-#define _PGMT_WB		(1UL << PG_uncached | 1UL << PG_arch_1)
+#define _PGMT_WT		(1UL << PG_uncached | 1UL << PG_arch_1)
 #define _PGMT_MASK		(1UL << PG_uncached | 1UL << PG_arch_1)
 #define _PGMT_CLEAR_MASK	(~_PGMT_MASK)
 
@@ -89,14 +93,14 @@ static inline enum page_cache_mode get_page_memtype(struct page *pg)
 {
 	unsigned long pg_flags = pg->flags & _PGMT_MASK;
 
-	if (pg_flags == _PGMT_DEFAULT)
-		return -1;
+	if (pg_flags == _PGMT_WB)
+		return _PAGE_CACHE_MODE_WB;
 	else if (pg_flags == _PGMT_WC)
 		return _PAGE_CACHE_MODE_WC;
 	else if (pg_flags == _PGMT_UC_MINUS)
 		return _PAGE_CACHE_MODE_UC_MINUS;
 	else
-		return _PAGE_CACHE_MODE_WB;
+		return _PAGE_CACHE_MODE_WT;
 }
 
 static inline void set_page_memtype(struct page *pg,
@@ -113,11 +117,12 @@ static inline void set_page_memtype(struct page *pg,
 	case _PAGE_CACHE_MODE_UC_MINUS:
 		memtype_flags = _PGMT_UC_MINUS;
 		break;
-	case _PAGE_CACHE_MODE_WB:
-		memtype_flags = _PGMT_WB;
+	case _PAGE_CACHE_MODE_WT:
+		memtype_flags = _PGMT_WT;
 		break;
+	case _PAGE_CACHE_MODE_WB:
 	default:
-		memtype_flags = _PGMT_DEFAULT;
+		memtype_flags = _PGMT_WB;
 		break;
 	}
 
@@ -401,8 +406,11 @@ static int pat_pagerange_is_ram(resource_size_t start, resource_size_t end)
 
 /*
  * For RAM pages, we use page flags to mark the pages with appropriate type.
- * The page flags are limited to three types, WB, WC and UC-.
- * WT and WP requests fail with -EINVAL, and UC gets redirected to UC-.
+ * The page flags are limited to four types, WB (default), WC, WT and UC-.
+ * WP request fails with -EINVAL, and UC gets redirected to UC-.  Setting
+ * a new memory type is only allowed to a page mapped with the default WB
+ * type.
+ *
  * Here we do two pass:
  * - Find the memtype of all the pages in the range, look for any conflicts
  * - In case of no conflicts, set the new memtype for pages in the range
@@ -414,8 +422,7 @@ static int reserve_ram_pages_type(u64 start, u64 end,
 	struct page *page;
 	u64 pfn;
 
-	if ((req_type == _PAGE_CACHE_MODE_WT) ||
-	    (req_type == _PAGE_CACHE_MODE_WP)) {
+	if (req_type == _PAGE_CACHE_MODE_WP) {
 		if (new_type)
 			*new_type = _PAGE_CACHE_MODE_UC_MINUS;
 		return -EINVAL;
@@ -432,7 +439,7 @@ static int reserve_ram_pages_type(u64 start, u64 end,
 
 		page = pfn_to_page(pfn);
 		type = get_page_memtype(page);
-		if (type != -1) {
+		if (type != _PAGE_CACHE_MODE_WB) {
 			pr_info("x86/PAT: reserve_ram_pages_type failed [mem %#010Lx-%#010Lx], track 0x%x, req 0x%x\n",
 				start, end - 1, type, req_type);
 			if (new_type)
@@ -459,7 +466,7 @@ static int free_ram_pages_type(u64 start, u64 end)
 
 	for (pfn = (start >> PAGE_SHIFT); pfn < (end >> PAGE_SHIFT); ++pfn) {
 		page = pfn_to_page(pfn);
-		set_page_memtype(page, -1);
+		set_page_memtype(page, _PAGE_CACHE_MODE_WB);
 	}
 	return 0;
 }
@@ -600,7 +607,7 @@ int free_memtype(u64 start, u64 end)
  * Only to be called when PAT is enabled
  *
  * Returns _PAGE_CACHE_MODE_WB, _PAGE_CACHE_MODE_WC, _PAGE_CACHE_MODE_UC_MINUS
- * or _PAGE_CACHE_MODE_UC
+ * or _PAGE_CACHE_MODE_WT.
  */
 static enum page_cache_mode lookup_memtype(u64 paddr)
 {
@@ -612,16 +619,9 @@ static enum page_cache_mode lookup_memtype(u64 paddr)
 
 	if (pat_pagerange_is_ram(paddr, paddr + PAGE_SIZE)) {
 		struct page *page;
-		page = pfn_to_page(paddr >> PAGE_SHIFT);
-		rettype = get_page_memtype(page);
-		/*
-		 * -1 from get_page_memtype() implies RAM page is in its
-		 * default state and not reserved, and hence of type WB
-		 */
-		if (rettype == -1)
-			rettype = _PAGE_CACHE_MODE_WB;
 
-		return rettype;
+		page = pfn_to_page(paddr >> PAGE_SHIFT);
+		return get_page_memtype(page);
 	}
 
 	spin_lock(&memtype_lock);

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 09/10] x86, mm: Add set_memory_wt() for WT
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (7 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 08/10] x86, mm, asm: Add WT support to set_page_memtype() Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 10/10] drivers/block/pmem: Map NVDIMM with ioremap_wt() Toshi Kani
  2015-06-02 16:21 ` [PATCH v12 0/10] Support Write-Through mapping on x86 Luis R. Rodriguez
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

Now that reserve_ram_pages_type() accepts the WT type, this
patch adds set_memory_wt(), set_memory_array_wt() and
set_pages_array_wt() for setting the WT type to the regular
memory.

ioremap_change_attr() is also extended to accept the WT type.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
---
 Documentation/x86/pat.txt         |    9 ++++-
 arch/x86/include/asm/cacheflush.h |    6 +++-
 arch/x86/mm/ioremap.c             |    3 ++
 arch/x86/mm/pageattr.c            |   62 +++++++++++++++++++++++++++++--------
 4 files changed, 63 insertions(+), 17 deletions(-)

diff --git a/Documentation/x86/pat.txt b/Documentation/x86/pat.txt
index db0de6c..54944c7 100644
--- a/Documentation/x86/pat.txt
+++ b/Documentation/x86/pat.txt
@@ -48,6 +48,9 @@ set_memory_uc          |    UC-   |    --      |       --         |
 set_memory_wc          |    WC    |    --      |       --         |
  set_memory_wb         |          |            |                  |
                        |          |            |                  |
+set_memory_wt          |    WT    |    --      |       --         |
+ set_memory_wb         |          |            |                  |
+                       |          |            |                  |
 pci sysfs resource     |    --    |    --      |       UC-        |
                        |          |            |                  |
 pci sysfs resource_wc  |    --    |    --      |       WC         |
@@ -150,8 +153,8 @@ can be more restrictive, in case of any existing aliasing for that address.
 For example: If there is an existing uncached mapping, a new ioremap_wc can
 return uncached mapping in place of write-combine requested.
 
-set_memory_[uc|wc] and set_memory_wb should be used in pairs, where driver will
-first make a region uc or wc and switch it back to wb after use.
+set_memory_[uc|wc|wt] and set_memory_wb should be used in pairs, where driver
+will first make a region uc, wc or wt and switch it back to wb after use.
 
 Over time writes to /proc/mtrr will be deprecated in favor of using PAT based
 interfaces. Users writing to /proc/mtrr are suggested to use above interfaces.
@@ -159,7 +162,7 @@ interfaces. Users writing to /proc/mtrr are suggested to use above interfaces.
 Drivers should use ioremap_[uc|wc] to access PCI BARs with [uc|wc] access
 types.
 
-Drivers should use set_memory_[uc|wc] to set access type for RAM ranges.
+Drivers should use set_memory_[uc|wc|wt] to set access type for RAM ranges.
 
 
 PAT debugging
diff --git a/arch/x86/include/asm/cacheflush.h b/arch/x86/include/asm/cacheflush.h
index 47c8e32..b6f7457 100644
--- a/arch/x86/include/asm/cacheflush.h
+++ b/arch/x86/include/asm/cacheflush.h
@@ -8,7 +8,7 @@
 /*
  * The set_memory_* API can be used to change various attributes of a virtual
  * address range. The attributes include:
- * Cachability   : UnCached, WriteCombining, WriteBack
+ * Cachability   : UnCached, WriteCombining, WriteThrough, WriteBack
  * Executability : eXeutable, NoteXecutable
  * Read/Write    : ReadOnly, ReadWrite
  * Presence      : NotPresent
@@ -35,9 +35,11 @@
 
 int _set_memory_uc(unsigned long addr, int numpages);
 int _set_memory_wc(unsigned long addr, int numpages);
+int _set_memory_wt(unsigned long addr, int numpages);
 int _set_memory_wb(unsigned long addr, int numpages);
 int set_memory_uc(unsigned long addr, int numpages);
 int set_memory_wc(unsigned long addr, int numpages);
+int set_memory_wt(unsigned long addr, int numpages);
 int set_memory_wb(unsigned long addr, int numpages);
 int set_memory_x(unsigned long addr, int numpages);
 int set_memory_nx(unsigned long addr, int numpages);
@@ -48,10 +50,12 @@ int set_memory_4k(unsigned long addr, int numpages);
 
 int set_memory_array_uc(unsigned long *addr, int addrinarray);
 int set_memory_array_wc(unsigned long *addr, int addrinarray);
+int set_memory_array_wt(unsigned long *addr, int addrinarray);
 int set_memory_array_wb(unsigned long *addr, int addrinarray);
 
 int set_pages_array_uc(struct page **pages, int addrinarray);
 int set_pages_array_wc(struct page **pages, int addrinarray);
+int set_pages_array_wt(struct page **pages, int addrinarray);
 int set_pages_array_wb(struct page **pages, int addrinarray);
 
 /*
diff --git a/arch/x86/mm/ioremap.c b/arch/x86/mm/ioremap.c
index 07cd46a..8405c0c 100644
--- a/arch/x86/mm/ioremap.c
+++ b/arch/x86/mm/ioremap.c
@@ -42,6 +42,9 @@ int ioremap_change_attr(unsigned long vaddr, unsigned long size,
 	case _PAGE_CACHE_MODE_WC:
 		err = _set_memory_wc(vaddr, nrpages);
 		break;
+	case _PAGE_CACHE_MODE_WT:
+		err = _set_memory_wt(vaddr, nrpages);
+		break;
 	case _PAGE_CACHE_MODE_WB:
 		err = _set_memory_wb(vaddr, nrpages);
 		break;
diff --git a/arch/x86/mm/pageattr.c b/arch/x86/mm/pageattr.c
index 94aae76..d908f19 100644
--- a/arch/x86/mm/pageattr.c
+++ b/arch/x86/mm/pageattr.c
@@ -1502,12 +1502,10 @@ EXPORT_SYMBOL(set_memory_uc);
 static int _set_memory_array(unsigned long *addr, int addrinarray,
 		enum page_cache_mode new_type)
 {
+	enum page_cache_mode set_type;
 	int i, j;
 	int ret;
 
-	/*
-	 * for now UC MINUS. see comments in ioremap_nocache()
-	 */
 	for (i = 0; i < addrinarray; i++) {
 		ret = reserve_memtype(__pa(addr[i]), __pa(addr[i]) + PAGE_SIZE,
 					new_type, NULL);
@@ -1515,9 +1513,12 @@ static int _set_memory_array(unsigned long *addr, int addrinarray,
 			goto out_free;
 	}
 
+	/* If WC, set to UC- first and then WC */
+	set_type = (new_type == _PAGE_CACHE_MODE_WC) ?
+				_PAGE_CACHE_MODE_UC_MINUS : new_type;
+
 	ret = change_page_attr_set(addr, addrinarray,
-				   cachemode2pgprot(_PAGE_CACHE_MODE_UC_MINUS),
-				   1);
+				   cachemode2pgprot(set_type), 1);
 
 	if (!ret && new_type == _PAGE_CACHE_MODE_WC)
 		ret = change_page_attr_set_clr(addr, addrinarray,
@@ -1549,6 +1550,12 @@ int set_memory_array_wc(unsigned long *addr, int addrinarray)
 }
 EXPORT_SYMBOL(set_memory_array_wc);
 
+int set_memory_array_wt(unsigned long *addr, int addrinarray)
+{
+	return _set_memory_array(addr, addrinarray, _PAGE_CACHE_MODE_WT);
+}
+EXPORT_SYMBOL_GPL(set_memory_array_wt);
+
 int _set_memory_wc(unsigned long addr, int numpages)
 {
 	int ret;
@@ -1574,21 +1581,39 @@ int set_memory_wc(unsigned long addr, int numpages)
 	ret = reserve_memtype(__pa(addr), __pa(addr) + numpages * PAGE_SIZE,
 		_PAGE_CACHE_MODE_WC, NULL);
 	if (ret)
-		goto out_err;
+		return ret;
 
 	ret = _set_memory_wc(addr, numpages);
 	if (ret)
-		goto out_free;
+		free_memtype(__pa(addr), __pa(addr) + numpages * PAGE_SIZE);
 
-	return 0;
-
-out_free:
-	free_memtype(__pa(addr), __pa(addr) + numpages * PAGE_SIZE);
-out_err:
 	return ret;
 }
 EXPORT_SYMBOL(set_memory_wc);
 
+int _set_memory_wt(unsigned long addr, int numpages)
+{
+	return change_page_attr_set(&addr, numpages,
+				    cachemode2pgprot(_PAGE_CACHE_MODE_WT), 0);
+}
+
+int set_memory_wt(unsigned long addr, int numpages)
+{
+	int ret;
+
+	ret = reserve_memtype(__pa(addr), __pa(addr) + numpages * PAGE_SIZE,
+			      _PAGE_CACHE_MODE_WT, NULL);
+	if (ret)
+		return ret;
+
+	ret = _set_memory_wt(addr, numpages);
+	if (ret)
+		free_memtype(__pa(addr), __pa(addr) + numpages * PAGE_SIZE);
+
+	return ret;
+}
+EXPORT_SYMBOL_GPL(set_memory_wt);
+
 int _set_memory_wb(unsigned long addr, int numpages)
 {
 	/* WB cache mode is hard wired to all cache attribute bits being 0 */
@@ -1679,6 +1704,7 @@ static int _set_pages_array(struct page **pages, int addrinarray,
 {
 	unsigned long start;
 	unsigned long end;
+	enum page_cache_mode set_type;
 	int i;
 	int free_idx;
 	int ret;
@@ -1692,8 +1718,12 @@ static int _set_pages_array(struct page **pages, int addrinarray,
 			goto err_out;
 	}
 
+	/* If WC, set to UC- first and then WC */
+	set_type = (new_type == _PAGE_CACHE_MODE_WC) ?
+				_PAGE_CACHE_MODE_UC_MINUS : new_type;
+
 	ret = cpa_set_pages_array(pages, addrinarray,
-			cachemode2pgprot(_PAGE_CACHE_MODE_UC_MINUS));
+				  cachemode2pgprot(set_type));
 	if (!ret && new_type == _PAGE_CACHE_MODE_WC)
 		ret = change_page_attr_set_clr(NULL, addrinarray,
 					       cachemode2pgprot(
@@ -1727,6 +1757,12 @@ int set_pages_array_wc(struct page **pages, int addrinarray)
 }
 EXPORT_SYMBOL(set_pages_array_wc);
 
+int set_pages_array_wt(struct page **pages, int addrinarray)
+{
+	return _set_pages_array(pages, addrinarray, _PAGE_CACHE_MODE_WT);
+}
+EXPORT_SYMBOL_GPL(set_pages_array_wt);
+
 int set_pages_wb(struct page *page, int numpages)
 {
 	unsigned long addr = (unsigned long)page_address(page);

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 10/10] drivers/block/pmem: Map NVDIMM with ioremap_wt()
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (8 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 09/10] x86, mm: Add set_memory_wt() for WT Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-02 16:21 ` [PATCH v12 0/10] Support Write-Through mapping on x86 Luis R. Rodriguez
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

The pmem driver maps NVDIMM with ioremap_nocache() as we cannot
write back the contents of the CPU caches in case of a crash.

This patch changes to use ioremap_wt(), which provides uncached
writes but cached reads, for improving read performance.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Acked-by: Dan Williams <dan.j.williams@intel.com>
---
 drivers/block/pmem.c |    4 ++--
 1 file changed, 2 insertions(+), 2 deletions(-)

diff --git a/drivers/block/pmem.c b/drivers/block/pmem.c
index eabf4a8..095dfaa 100644
--- a/drivers/block/pmem.c
+++ b/drivers/block/pmem.c
@@ -139,11 +139,11 @@ static struct pmem_device *pmem_alloc(struct device *dev, struct resource *res)
 	}
 
 	/*
-	 * Map the memory as non-cachable, as we can't write back the contents
+	 * Map the memory as write-through, as we can't write back the contents
 	 * of the CPU caches in case of a crash.
 	 */
 	err = -ENOMEM;
-	pmem->virt_addr = ioremap_nocache(pmem->phys_addr, pmem->size);
+	pmem->virt_addr = ioremap_wt(pmem->phys_addr, pmem->size);
 	if (!pmem->virt_addr)
 		goto out_release_region;
 

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough()
  2015-06-01 19:36 ` [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough() Toshi Kani
@ 2015-06-02  7:06   ` Geert Uytterhoeven
  0 siblings, 0 replies; 17+ messages in thread
From: Geert Uytterhoeven @ 2015-06-02  7:06 UTC (permalink / raw)
  To: Toshi Kani
  Cc: Borislav Petkov, H. Peter Anvin, Thomas Gleixner, Ingo Molnar,
	Andrew Morton, Arnd Bergmann, Linux MM, linux-kernel,
	the arch/x86 maintainers, linux-nvdimm, jgross, stefan.bader,
	Andy Lutomirski, Henrique de Moraes Holschuh, yigal,
	Konrad Rzeszutek Wilk, Elliott, Luis R. Rodriguez,
	Christoph Hellwig

On Mon, Jun 1, 2015 at 9:36 PM, Toshi Kani <toshi.kani@hp.com> wrote:
> This patch removes the callers of ioremap_writethrough() by
> replacing them with ioremap_wt() in three drivers under
> drivers/video/fbdev.  It then removes ioremap_writethrough()
> defined in some architecture's asm/io.h, frv, m68k, microblaze,
> and tile.
>
> Signed-off-by: Toshi Kani <toshi.kani@hp.com>
> ---
>  arch/frv/include/asm/io.h        |    5 -----
>  arch/m68k/include/asm/io_mm.h    |    5 -----
>  arch/m68k/include/asm/io_no.h    |    4 ----
>  arch/microblaze/include/asm/io.h |    1 -
>  arch/tile/include/asm/io.h       |    1 -
>  drivers/video/fbdev/amifb.c      |    4 ++--
>  drivers/video/fbdev/atafb.c      |    3 +--
>  drivers/video/fbdev/hpfb.c       |    4 ++--

For the m68k and amifb/atafb/hpfb changes:
Acked-by: Geert Uytterhoeven <geert@linux-m68k.org>

Gr{oetje,eeting}s,

                        Geert

diff --git a/arch/x86/mm/pat.c b/arch/x86/mm/pat.c
index f89e460..83af01a 100644
--- a/arch/x86/mm/pat.c
+++ b/arch/x86/mm/pat.c
@@ -235,6 +235,7 @@ static void pat_ap_init(u64 pat)
 void pat_init(void)
 {
 	u64 pat;
+	struct cpuinfo_x86 *c = &boot_cpu_data;
 
 	if (!pat_enabled()) {
 		/*
@@ -259,21 +260,61 @@ void pat_init(void)
 		 */
 		pat = PAT(0, WB) | PAT(1, WT) | PAT(2, UC_MINUS) | PAT(3, UC) |
 		      PAT(4, WB) | PAT(5, WT) | PAT(6, UC_MINUS) | PAT(7, UC);
-	} else {
+
+	} else if ((c->x86_vendor == X86_VENDOR_INTEL) &&
+		   (((c->x86 == 0x6) && (c->x86_model <= 0xd)) ||
+		    ((c->x86 == 0xf) && (c->x86_model <= 0x6)))) {
 		/*
-		 * PTE encoding used in Linux:
+		 * PAT support with the lower four entries. Intel Pentium 2,
+		 * 3, M, and 4 are affected by PAT errata, which makes the
+		 * upper four entries unusable.  We do not use the upper four
+		 * entries for all the affected processor families for safe.
+		 *
+		 *  PTE encoding used in Linux:
 		 *      PAT
 		 *      |PCD
 		 *      ||PWT
 		 *      |||
 		 *      000 WB          _PAGE_CACHE_WB
 		 *      001 WC          _PAGE_CACHE_WC
 		 *      010 UC-         _PAGE_CACHE_UC_MINUS
 		 *      011 UC          _PAGE_CACHE_UC
 		 * PAT bit unused
+		 *
+		 * NOTE: When WT or WP is used, it is redirected to UC- per
+		 * the default setup in __cachemode2pte_tbl[].
 		 */
 		pat = PAT(0, WB) | PAT(1, WC) | PAT(2, UC_MINUS) | PAT(3, UC) |
 		      PAT(4, WB) | PAT(5, WC) | PAT(6, UC_MINUS) | PAT(7, UC);
+	} else {
+		/*
+		 * PAT full support.  We put WT in slot 7 to improve
+		 * robustness in the presence of errata that might cause
+		 * the high PAT bit to be ignored.  This way a buggy slot 7
+		 * access will hit slot 3, and slot 3 is UC, so at worst
+		 * we lose performance without causing a correctness issue.
+		 * Pentium 4 erratum N46 is an example of such an erratum,
+		 * although we try not to use PAT at all on affected CPUs.
+		 *
+		 *  PTE encoding used in Linux:
+		 *      PAT
+		 *      |PCD
+		 *      ||PWT  PAT
+		 *      |||    slot
+		 *      000    0    WB : _PAGE_CACHE_MODE_WB
+		 *      001    1    WC : _PAGE_CACHE_MODE_WC
+		 *      010    2    UC-: _PAGE_CACHE_MODE_UC_MINUS
+		 *      011    3    UC : _PAGE_CACHE_MODE_UC
+		 *      100    4    WB : Reserved
+		 *      101    5    WC : Reserved
+		 *      110    6    UC-: Reserved
+		 *      111    7    WT : _PAGE_CACHE_MODE_WT
+		 *
+		 * The reserved slots are unused, but mapped to their
+		 * corresponding types in the presence of PAT errata.
+		 */
+		pat = PAT(0, WB) | PAT(1, WC) | PAT(2, UC_MINUS) | PAT(3, UC) |
+		      PAT(4, WB) | PAT(5, WC) | PAT(6, UC_MINUS) | PAT(7, WT);
 	}
 
 	if (!boot_cpu_done) {

^ permalink raw reply related	[flat|nested] 17+ messages in thread

* [PATCH v12 2/10] x86, mm, pat: Change reserve_memtype() for WT
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 1/10] x86, mm, pat: Set WT to PA7 slot of PAT MSR Toshi Kani
@ 2015-06-01 19:36 ` Toshi Kani
  2015-06-01 19:36 ` [PATCH v12 3/10] x86, asm: Change is_new_memtype_allowed() " Toshi Kani
                   ` (8 subsequent siblings)
  10 siblings, 0 replies; 17+ messages in thread
From: Toshi Kani @ 2015-06-01 19:36 UTC (permalink / raw)
  To: bp, hpa, tglx, mingo, akpm, arnd
  Cc: linux-mm, linux-kernel, x86, linux-nvdimm, jgross, stefan.bader,
	luto, hmh, yigal, konrad.wilk, Elliott, mcgrof, hch, Toshi Kani

When a RAM range is requested to reserve_memtype(), it calls
reserve_ram_pages_type() to verify the requested type.
reserve_ram_pages_type() is changed to fail WT and WP requests
with -EINVAL since set_page_memtype() is limited to handle
three types, WB, WC and UC-.

Signed-off-by: Toshi Kani <toshi.kani@hp.com>
Reviewed-by: Konrad Rzeszutek Wilk <konrad.wilk@oracle.com>
Reviewed-by: Thomas Gleixner <tglx@linutronix.de>
---
Patch 8/10 enhances set_page_memtype() to support WT.
---
 arch/x86/mm/pat.c |   10 ++++++++++
 1 file changed, 10 insertions(+)
@@ -259,21 +260,61 @@ void pat_init(void)
 		 *      PAT
 		 *      |PCD
-		 *      ||PWT
-		 *      |||
-		 *      000 WB          _PAGE_CACHE_WB
-		 *      001 WC          _PAGE_CACHE_WC
-		 *      010 UC-         _PAGE_CACHE_UC_MINUS
-		 *      011 UC          _PAGE_CACHE_UC
+		 *      ||PWT  PAT
+		 *      |||    slot
+		 *      000    0    WB : _PAGE_CACHE_MODE_WB
+		 *      001    1    WC : _PAGE_CACHE_MODE_WC
+		 *      010    2    UC-: _PAGE_CACHE_MODE_UC_MINUS
+		 *      011    3    UC : _PAGE_CACHE_MODE_UC
 		 * PAT bit unused

--
Geert Uytterhoeven -- There's lots of Linux beyond ia32 -- geert@linux-m68k.org

In personal conversations with technical people, I call myself a hacker. But
when I'm talking to journalists I just say "programmer" or something like that.
                                -- Linus Torvalds

^ permalink raw reply	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 0/10] Support Write-Through mapping on x86
  2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
                   ` (9 preceding siblings ...)
  2015-06-01 19:36 ` [PATCH v12 10/10] drivers/block/pmem: Map NVDIMM with ioremap_wt() Toshi Kani
@ 2015-06-02 16:21 ` Luis R. Rodriguez
  2015-06-02 18:46   ` Toshi Kani
  10 siblings, 1 reply; 17+ messages in thread
From: Luis R. Rodriguez @ 2015-06-02 16:21 UTC (permalink / raw)
  To: Toshi Kani
  Cc: bp, hpa, tglx, mingo, akpm, arnd, linux-mm, linux-kernel, x86,
	linux-nvdimm, jgross, stefan.bader, luto, hmh, yigal,
	konrad.wilk, Elliott, hch

On Mon, Jun 01, 2015 at 01:36:23PM -0600, Toshi Kani wrote:
> This patchset adds support of Write-Through (WT) mapping on x86.
> The study below shows that using WT mapping may be useful for
> non-volatile memory.
> 
> http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf
> 
> The patchset consists of the following changes.
>  - Patch 1/10 to 6/10 add ioremap_wt()
>  - Patch 7/10 adds pgprot_writethrough()
>  - Patch 8/10 to 9/10 add set_memory_wt()
>  - Patch 10/10 changes the pmem driver to call ioremap_wt()
> 
> All new/modified interfaces have been tested.
> 
> The patchset is based on:
> git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2

While at it can you also look at:

mcgrof@ergon ~/linux-next (git::master)$ git grep -4 "writethrough" drivers/infiniband/

drivers/infiniband/hw/ipath/ipath_driver.c-
drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_pcirev = pdev->revision;
drivers/infiniband/hw/ipath/ipath_driver.c-
drivers/infiniband/hw/ipath/ipath_driver.c-#if defined(__powerpc__)
drivers/infiniband/hw/ipath/ipath_driver.c:     /* There isn't a generic way to specify writethrough mappings */
drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_kregbase = __ioremap(addr, len,
drivers/infiniband/hw/ipath/ipath_driver.c-             (_PAGE_NO_CACHE|_PAGE_WRITETHRU));
drivers/infiniband/hw/ipath/ipath_driver.c-#else
drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_kregbase = ioremap_nocache(addr, len);
--
drivers/infiniband/hw/ipath/ipath_file_ops.c-
drivers/infiniband/hw/ipath/ipath_file_ops.c-   phys = dd->ipath_physaddr + piobufs;
drivers/infiniband/hw/ipath/ipath_file_ops.c-
drivers/infiniband/hw/ipath/ipath_file_ops.c-#if defined(__powerpc__)
drivers/infiniband/hw/ipath/ipath_file_ops.c:   /* There isn't a generic way to specify writethrough mappings */
drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE;
drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) |= _PAGE_WRITETHRU;
drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) &= ~_PAGE_GUARDED;
drivers/infiniband/hw/ipath/ipath_file_ops.c-#endif
--
drivers/infiniband/hw/qib/qib_file_ops.c-
drivers/infiniband/hw/qib/qib_file_ops.c-       phys = dd->physaddr + piobufs;
drivers/infiniband/hw/qib/qib_file_ops.c-
drivers/infiniband/hw/qib/qib_file_ops.c-#if defined(__powerpc__)
drivers/infiniband/hw/qib/qib_file_ops.c:       /* There isn't a generic way to specify writethrough mappings */
drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE;
drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) |= _PAGE_WRITETHRU;
drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) &= ~_PAGE_GUARDED;
drivers/infiniband/hw/qib/qib_file_ops.c-#endif
--
drivers/infiniband/hw/qib/qib_pcie.c-   addr = pci_resource_start(pdev, 0);
drivers/infiniband/hw/qib/qib_pcie.c-   len = pci_resource_len(pdev, 0);
drivers/infiniband/hw/qib/qib_pcie.c-
drivers/infiniband/hw/qib/qib_pcie.c-#if defined(__powerpc__)
drivers/infiniband/hw/qib/qib_pcie.c:   /* There isn't a generic way to specify writethrough mappings */
drivers/infiniband/hw/qib/qib_pcie.c-   dd->kregbase = __ioremap(addr, len, _PAGE_NO_CACHE | _PAGE_WRITETHRU);
drivers/infiniband/hw/qib/qib_pcie.c-#else
drivers/infiniband/hw/qib/qib_pcie.c-   dd->kregbase = ioremap_nocache(addr, len);
drivers/infiniband/hw/qib/qib_pcie.c-#endif

  Luis

^ permalink raw reply	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 0/10] Support Write-Through mapping on x86
  2015-06-02 16:21 ` [PATCH v12 0/10] Support Write-Through mapping on x86 Luis R. Rodriguez
@ 2015-06-02 18:46   ` Toshi Kani
  2015-06-02 19:43     ` Luis R. Rodriguez
  0 siblings, 1 reply; 17+ messages in thread
From: Toshi Kani @ 2015-06-02 18:46 UTC (permalink / raw)
  To: Luis R. Rodriguez
  Cc: bp, hpa, tglx, mingo, akpm, arnd, linux-mm, linux-kernel, x86,
	linux-nvdimm, jgross, stefan.bader, luto, hmh, yigal,
	konrad.wilk, Elliott, hch

On Tue, 2015-06-02 at 18:21 +0200, Luis R. Rodriguez wrote:
> On Mon, Jun 01, 2015 at 01:36:23PM -0600, Toshi Kani wrote:
> > This patchset adds support of Write-Through (WT) mapping on x86.
> > The study below shows that using WT mapping may be useful for
> > non-volatile memory.
> > 
> > http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf
> > 
> > The patchset consists of the following changes.
> >  - Patch 1/10 to 6/10 add ioremap_wt()
> >  - Patch 7/10 adds pgprot_writethrough()
> >  - Patch 8/10 to 9/10 add set_memory_wt()
> >  - Patch 10/10 changes the pmem driver to call ioremap_wt()
> > 
> > All new/modified interfaces have been tested.
> > 
> > The patchset is based on:
> > git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2
> 
> While at it can you also look at:
>
> mcgrof@ergon ~/linux-next (git::master)$ git grep -4 "writethrough" drivers/infiniband/

Thanks for checking this.  The inifiniband driver uses WT mappings on
powerpc without proper WT interfaces defined.  They can be cleaned up by
a separate patch series to support WT on powerpc in the same way after
this patchset (support WT on x86) is settled.

> drivers/infiniband/hw/ipath/ipath_driver.c-
> drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_pcirev = pdev->revision;
> drivers/infiniband/hw/ipath/ipath_driver.c-
> drivers/infiniband/hw/ipath/ipath_driver.c-#if defined(__powerpc__)
> drivers/infiniband/hw/ipath/ipath_driver.c:     /* There isn't a generic way to specify writethrough mappings */
> drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_kregbase = __ioremap(addr, len,
> drivers/infiniband/hw/ipath/ipath_driver.c-             (_PAGE_NO_CACHE|_PAGE_WRITETHRU));
> drivers/infiniband/hw/ipath/ipath_driver.c-#else
> drivers/infiniband/hw/ipath/ipath_driver.c-     dd->ipath_kregbase = ioremap_nocache(addr, len);

Once powerpc defines ioremap_wt(), this driver can simply call
ioremap_wt() without the ifdef.  ioremap_wt() is defined as
ioremap_nocache() on the architectures without WT support.

> drivers/infiniband/hw/ipath/ipath_file_ops.c-
> drivers/infiniband/hw/ipath/ipath_file_ops.c-   phys = dd->ipath_physaddr + piobufs;
> drivers/infiniband/hw/ipath/ipath_file_ops.c-
> drivers/infiniband/hw/ipath/ipath_file_ops.c-#if defined(__powerpc__)
> drivers/infiniband/hw/ipath/ipath_file_ops.c:   /* There isn't a generic way to specify writethrough mappings */
> drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE;
> drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) |= _PAGE_WRITETHRU;
> drivers/infiniband/hw/ipath/ipath_file_ops.c-   pgprot_val(vma->vm_page_prot) &= ~_PAGE_GUARDED;
> drivers/infiniband/hw/ipath/ipath_file_ops.c-#endif

Once powerpc defines pgprot_writethrough(), this driver can use it
without the ifdef.  pgprot_writethrough() is defined as
pgprot_noncached() on the architectures without WT support.

> drivers/infiniband/hw/qib/qib_file_ops.c-
> drivers/infiniband/hw/qib/qib_file_ops.c-       phys = dd->physaddr + piobufs;
> drivers/infiniband/hw/qib/qib_file_ops.c-
> drivers/infiniband/hw/qib/qib_file_ops.c-#if defined(__powerpc__)
> drivers/infiniband/hw/qib/qib_file_ops.c:       /* There isn't a generic way to specify writethrough mappings */
> drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) |= _PAGE_NO_CACHE;
> drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) |= _PAGE_WRITETHRU;
> drivers/infiniband/hw/qib/qib_file_ops.c-       pgprot_val(vma->vm_page_prot) &= ~_PAGE_GUARDED;
> drivers/infiniband/hw/qib/qib_file_ops.c-#endif

Same as above.

> drivers/infiniband/hw/qib/qib_pcie.c-   addr = pci_resource_start(pdev, 0);
> drivers/infiniband/hw/qib/qib_pcie.c-   len = pci_resource_len(pdev, 0);
> drivers/infiniband/hw/qib/qib_pcie.c-
> drivers/infiniband/hw/qib/qib_pcie.c-#if defined(__powerpc__)
> drivers/infiniband/hw/qib/qib_pcie.c:   /* There isn't a generic way to specify writethrough mappings */
> drivers/infiniband/hw/qib/qib_pcie.c-   dd->kregbase = __ioremap(addr, len, _PAGE_NO_CACHE | _PAGE_WRITETHRU);
> drivers/infiniband/hw/qib/qib_pcie.c-#else
> drivers/infiniband/hw/qib/qib_pcie.c-   dd->kregbase = ioremap_nocache(addr, len);
> drivers/infiniband/hw/qib/qib_pcie.c-#endif

Same as ipath_driver.c.

Thanks,
-Toshi


^ permalink raw reply	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 0/10] Support Write-Through mapping on x86
  2015-06-02 18:46   ` Toshi Kani
@ 2015-06-02 19:43     ` Luis R. Rodriguez
  2015-06-02 19:50       ` Andy Lutomirski
  0 siblings, 1 reply; 17+ messages in thread
From: Luis R. Rodriguez @ 2015-06-02 19:43 UTC (permalink / raw)
  To: Toshi Kani
  Cc: bp, hpa, tglx, mingo, akpm, arnd, linux-mm, linux-kernel, x86,
	linux-nvdimm, jgross, stefan.bader, luto, hmh, yigal,
	konrad.wilk, Elliott, hch

On Tue, Jun 02, 2015 at 12:46:48PM -0600, Toshi Kani wrote:
> On Tue, 2015-06-02 at 18:21 +0200, Luis R. Rodriguez wrote:
> > On Mon, Jun 01, 2015 at 01:36:23PM -0600, Toshi Kani wrote:
> > > This patchset adds support of Write-Through (WT) mapping on x86.
> > > The study below shows that using WT mapping may be useful for
> > > non-volatile memory.
> > > 
> > > http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf
> > > 
> > > The patchset consists of the following changes.
> > >  - Patch 1/10 to 6/10 add ioremap_wt()
> > >  - Patch 7/10 adds pgprot_writethrough()
> > >  - Patch 8/10 to 9/10 add set_memory_wt()
> > >  - Patch 10/10 changes the pmem driver to call ioremap_wt()
> > > 
> > > All new/modified interfaces have been tested.
> > > 
> > > The patchset is based on:
> > > git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2
> > 
> > While at it can you also look at:
> >
> > mcgrof@ergon ~/linux-next (git::master)$ git grep -4 "writethrough" drivers/infiniband/
> 
> Thanks for checking this.  The inifiniband driver uses WT mappings on
> powerpc without proper WT interfaces defined. 

Right.

>  They can be cleaned up by
> a separate patch series to support WT on powerpc in the same way after
> this patchset (support WT on x86) is settled.

Who's gonna do that work though? How much work is it ? Is it too much to ask
to roll it in this series?

  Luis

^ permalink raw reply	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 0/10] Support Write-Through mapping on x86
  2015-06-02 19:43     ` Luis R. Rodriguez
@ 2015-06-02 19:50       ` Andy Lutomirski
  2015-06-02 20:01         ` Luis R. Rodriguez
  0 siblings, 1 reply; 17+ messages in thread
From: Andy Lutomirski @ 2015-06-02 19:50 UTC (permalink / raw)
  To: Luis R. Rodriguez
  Cc: Toshi Kani, Borislav Petkov, H. Peter Anvin, Thomas Gleixner,
	Ingo Molnar, Andrew Morton, Arnd Bergmann, linux-mm,
	linux-kernel, X86 ML, linux-nvdimm, Juergen Gross, Stefan Bader,
	Henrique de Moraes Holschuh, Yigal Korman, Konrad Rzeszutek Wilk,
	Elliott, Robert (Server Storage),
	Christoph Hellwig

On Tue, Jun 2, 2015 at 12:43 PM, Luis R. Rodriguez <mcgrof@suse.com> wrote:
> On Tue, Jun 02, 2015 at 12:46:48PM -0600, Toshi Kani wrote:
>> On Tue, 2015-06-02 at 18:21 +0200, Luis R. Rodriguez wrote:
>> > On Mon, Jun 01, 2015 at 01:36:23PM -0600, Toshi Kani wrote:
>> > > This patchset adds support of Write-Through (WT) mapping on x86.
>> > > The study below shows that using WT mapping may be useful for
>> > > non-volatile memory.
>> > >
>> > > http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf
>> > >
>> > > The patchset consists of the following changes.
>> > >  - Patch 1/10 to 6/10 add ioremap_wt()
>> > >  - Patch 7/10 adds pgprot_writethrough()
>> > >  - Patch 8/10 to 9/10 add set_memory_wt()
>> > >  - Patch 10/10 changes the pmem driver to call ioremap_wt()
>> > >
>> > > All new/modified interfaces have been tested.
>> > >
>> > > The patchset is based on:
>> > > git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2
>> >
>> > While at it can you also look at:
>> >
>> > mcgrof@ergon ~/linux-next (git::master)$ git grep -4 "writethrough" drivers/infiniband/
>>
>> Thanks for checking this.  The inifiniband driver uses WT mappings on
>> powerpc without proper WT interfaces defined.
>
> Right.
>
>>  They can be cleaned up by
>> a separate patch series to support WT on powerpc in the same way after
>> this patchset (support WT on x86) is settled.
>
> Who's gonna do that work though? How much work is it ? Is it too much to ask
> to roll it in this series?
>

I think the driver maintainers should do it.  For all I know,
something will go horribly wrong if those drivers suddenly start using
WT on x86.

--Andy

^ permalink raw reply	[flat|nested] 17+ messages in thread

* Re: [PATCH v12 0/10] Support Write-Through mapping on x86
  2015-06-02 19:50       ` Andy Lutomirski
@ 2015-06-02 20:01         ` Luis R. Rodriguez
  0 siblings, 0 replies; 17+ messages in thread
From: Luis R. Rodriguez @ 2015-06-02 20:01 UTC (permalink / raw)
  To: Andy Lutomirski, linux-rdma, Marciniszyn, Mike, Doug Ledford, roland
  Cc: Toshi Kani, Borislav Petkov, H. Peter Anvin, Thomas Gleixner,
	Ingo Molnar, Andrew Morton, Arnd Bergmann, linux-mm,
	linux-kernel, X86 ML, linux-nvdimm, Juergen Gross, Stefan Bader,
	Henrique de Moraes Holschuh, Yigal Korman, Konrad Rzeszutek Wilk,
	Elliott, Robert (Server Storage),
	Christoph Hellwig

On Tue, Jun 02, 2015 at 12:50:09PM -0700, Andy Lutomirski wrote:
> On Tue, Jun 2, 2015 at 12:43 PM, Luis R. Rodriguez <mcgrof@suse.com> wrote:
> > On Tue, Jun 02, 2015 at 12:46:48PM -0600, Toshi Kani wrote:
> >> On Tue, 2015-06-02 at 18:21 +0200, Luis R. Rodriguez wrote:
> >> > On Mon, Jun 01, 2015 at 01:36:23PM -0600, Toshi Kani wrote:
> >> > > This patchset adds support of Write-Through (WT) mapping on x86.
> >> > > The study below shows that using WT mapping may be useful for
> >> > > non-volatile memory.
> >> > >
> >> > > http://www.hpl.hp.com/techreports/2012/HPL-2012-236.pdf
> >> > >
> >> > > The patchset consists of the following changes.
> >> > >  - Patch 1/10 to 6/10 add ioremap_wt()
> >> > >  - Patch 7/10 adds pgprot_writethrough()
> >> > >  - Patch 8/10 to 9/10 add set_memory_wt()
> >> > >  - Patch 10/10 changes the pmem driver to call ioremap_wt()
> >> > >
> >> > > All new/modified interfaces have been tested.
> >> > >
> >> > > The patchset is based on:
> >> > > git://git.kernel.org/pub/scm/linux/kernel/git/bp/bp.git#tip-mm-2
> >> >
> >> > While at it can you also look at:
> >> >
> >> > mcgrof@ergon ~/linux-next (git::master)$ git grep -4 "writethrough" drivers/infiniband/
> >>
> >> Thanks for checking this.  The inifiniband driver uses WT mappings on
> >> powerpc without proper WT interfaces defined.
> >
> > Right.
> >
> >>  They can be cleaned up by
> >> a separate patch series to support WT on powerpc in the same way after
> >> this patchset (support WT on x86) is settled.
> >
> > Who's gonna do that work though? How much work is it ? Is it too much to ask
> > to roll it in this series?
> >
> 
> I think the driver maintainers should do it.  For all I know,
> something will go horribly wrong if those drivers suddenly start using
> WT on x86.

OK. Letting qib driver folks know.

 Luis

^ permalink raw reply	[flat|nested] 17+ messages in thread

end of thread, other threads:[~2015-06-02 20:02 UTC | newest]

Thread overview: 17+ messages (download: mbox.gz / follow: Atom feed)
-- links below jump to the message on this page --
2015-06-01 19:36 [PATCH v12 0/10] Support Write-Through mapping on x86 Toshi Kani
2015-06-01 19:36 ` [PATCH v12 1/10] x86, mm, pat: Set WT to PA7 slot of PAT MSR Toshi Kani
2015-06-01 19:36 ` [PATCH v12 2/10] x86, mm, pat: Change reserve_memtype() for WT Toshi Kani
2015-06-01 19:36 ` [PATCH v12 3/10] x86, asm: Change is_new_memtype_allowed() " Toshi Kani
2015-06-01 19:36 ` [PATCH v12 4/10] x86, mm, asm-gen: Add ioremap_wt() " Toshi Kani
2015-06-01 19:36 ` [PATCH v12 5/10] arch/*/asm/io.h: Add ioremap_wt() to all architectures Toshi Kani
2015-06-01 19:36 ` [PATCH v12 6/10] video/fbdev, asm/io.h: Remove ioremap_writethrough() Toshi Kani
2015-06-02  7:06   ` Geert Uytterhoeven
2015-06-01 19:36 ` [PATCH v12 7/10] x86, mm, pat: Add pgprot_writethrough() for WT Toshi Kani
2015-06-01 19:36 ` [PATCH v12 08/10] x86, mm, asm: Add WT support to set_page_memtype() Toshi Kani
2015-06-01 19:36 ` [PATCH v12 09/10] x86, mm: Add set_memory_wt() for WT Toshi Kani
2015-06-01 19:36 ` [PATCH v12 10/10] drivers/block/pmem: Map NVDIMM with ioremap_wt() Toshi Kani
2015-06-02 16:21 ` [PATCH v12 0/10] Support Write-Through mapping on x86 Luis R. Rodriguez
2015-06-02 18:46   ` Toshi Kani
2015-06-02 19:43     ` Luis R. Rodriguez
2015-06-02 19:50       ` Andy Lutomirski
2015-06-02 20:01         ` Luis R. Rodriguez

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).